try:
    from PySide2.QtCore import *
    from PySide2.QtUiTools import *
    from PySide2.QtGui import *
    from PySide2.QtWidgets import *
except:
    from PyQt5.QtCore import *
    from PyQt5.QtUiTools import *
    from PyQt5.QtGui import *
    from PyQt5.QtWidgets import *




def get_text_from_selected_items(tree_widget, column_index):
    """
    Returns the text of the selected treeWidgetItems as a list of string

    :param tree_widget: your QTreeWidget
    :param column_index: <int> number of the column you want to ge the text from
    :return:
    """
    text_list = []
    model_indexes = tree_widget.selectedIndexes()
    for model_index in model_indexes:
        item = tree_widget.itemFromIndex(model_index)

        text_list.append(item.text(column_index))

    return text_list

def get_item_with_text(tree_widget, text, column_index):
    """
    Returns a QTreeWidgetItem that matches the selected text for the selected column

    :param tree_widget: QTreeWidget you want to search in
    :param text: <string> text
    :param column_index: <int> column number
    :return: QTreeWidgetItem if found or None if not found
    """
    try:
        iterator = QTreeWidgetItemIterator(tree_widget)
        while iterator:
            if iterator.value().text(column_index) == text:
                return iterator.value()
            iterator +=1
    except:
        return None

    return None

def remove_items(tree_widget, items=[], selected=False):
    """
    Removes the passed items from the tree widget

    :param tree_widget: QTreeWidget to remove items from
    :param items: <list> items you want to remove
    :param selected: <bool> if set to True, will remove the selected items
    :return:
    """
    if selected == True:
        items = tree_widget.selectedItems()

    root = tree_widget.invisibleRootItem()
    for item in items:
        (item.parent() or root).removeChild(item)

def set_item_background_color(tree_widget_item, q_color, columns="all"):
    """
    Sets the background color of a QTreeWidget

    :param tree_widget_item: QTreeWidget you want to color
    :param q_color: <QColor> the color as a QColor (eg: QColor(255, 0, 0, 255)
    :param columns: <string> | <list> either the string "all" to color all columns or a list of ints with specific column numbers
    :return:
    """
    if columns == "all":
        for column in range(tree_widget_item.columnCount()):
            tree_widget_item.setBackground(column, q_color)
    else:
        for column in columns:
            tree_widget_item.setBackground(column, q_color)

def resize_columns(tree_widget, columns="all"):
    """
    Sets the column size to the minimal amount needed

    :param tree_widget: QTreeWidget you want to restructure
    :param columns: <string> | <list> either the string "all" or a list with specific column numbers
    :return:
    """
    if columns == "all":
        for column in range(tree_widget.columnCount()):
            tree_widget.resizeColumnToContents(column)
    else:
        for column in columns:
            tree_widget.resizeColumnToContents(column)

def get_all_top_level_items(tree_widget):
    """
    Returns all the top level items of the QTreeWidget

    :param tree_widget: your QTreeWidget
    :return: list of QTreeWidgetItems
    """

    items = []
    for index in range(tree_widget.topLevelItemCount()):
        items.append(tree_widget.topLevelItem(index))
    return items

def get_all_under_item(item):
    """
    Returns all the children of a specific QTreeWidgetItem

    :param item: <QTreeWidgetItem>
    :return: <list> QTreeWidgetItems
    """
    items = []
    for index in range(item.childCount()):
        items.append(item.child(index))
        items.extend(get_all_under_item(item.child(index)))
    return items

def get_all_items(tree_widget):
    """
    Returns all the items of a QTreeWidget

    :param tree_widget: your QTreeWidget
    :return: <list>  QTreeWidgetItems
    """

    all_items = []
    for i in range(tree_widget.topLevelItemCount()):
        top_level_item = tree_widget.topLevelItem(i)
        all_items.append(top_level_item)
        all_items.extend(get_all_under_item(top_level_item))
    return all_items


try:
    from PySide2.QtCore import *
    from PySide2.QtUiTools import *
    from PySide2.QtGui import *
    from PySide2.QtWidgets import *
except:
    from PyQt5.QtCore import *
    from PyQt5.QtUiTools import *
    from PyQt5.QtGui import *
    from PyQt5.QtWidgets import *


def add_items(list_widget, items, duplicates_allowed=False, string_split_character=None, clear=False):
    """
    Convenience function to add items to your list widget

    :param list_widget: your QListWidget
    :param items: <string> | <list> a single string item ("option"), a list (["option_1", "option_2"]), or a string that can be split on a character like a comma separated string ("option_1, option_2")
    :param duplicates_allowed: <bool> >whether or not to allow duplicates
    :param string_split_character: <string> split the string on this character (eg: ","), the result is the list of items that get added to the list widget
    :param clear: <bool> whether or not to clear the listWidget first
    :return:
    """
    if not isinstance(items, (list, tuple)):
        if isinstance(items, str) and string_split_character is not None and string_split_character in items:
            tmp_list = [part.strip() for part in items.split(string_split_character)]
            items = tmp_list
        elif isinstance(items, str) and items == "":
            return
        else:
            items = [items]

    if clear is True:
        list_widget.clear()

    for item in items:
        if duplicates_allowed is True:
            list_widget.addItem(item)
        else:
            if len(list_widget.findItems(item, Qt.MatchExactly)) == 0 and item is not None:
                list_widget.addItem(item)

def remove_items(list_widget, items=None, selected=False, string_split_character=None):
    """
    Convenience function to remove items from a listWidget

    :param list_widget: your QListWidget
    :param items: <string> | <list> a single string item ("option"), a list (["option_1", "option_2"]), or a string that can be split on a character like a comma separated string ("option_1, option_2")
    :param string_split_character: <string> split the string on this character (eg: ","), the result is the list of items that get added to the listWidget
    :param selected: <bool> if set to true, the deletion will only happen in the listWidgetItems that are currently selected
    :return:
    """
    if items is not None:
        if not isinstance(items, (list, tuple)):
            if isinstance(items, str) and string_split_character is not None and string_split_character in items:
                tmpList = [part.strip() for part in items.split(string_split_character)]
                items = tmpList
            else:
                items = [items]

    if selected:
        items = list_widget.selectedItems()

    for item in items:
        list_widget.takeItem(list_widget.row(item))

def get_all_items(list_widget, as_string=True):
    """
    Gets all the items in a listWidget as a list

    :param list_widget: your QListWidget
    :param as_string: <bool> if set to true, will return the text of the item. If set to false will return the actual QListWidgetItem
    :return: items of your QListWidget
    """
    items = []

    if as_string is True:
        for item in [list_widget.item(i).text() for i in
                     range(list_widget.count())]:
            if item is not None:
                items.append(item)
    else:
        for item in range(list_widget.count()):
            items.append(item)

    return items



try:
    from PySide2.QtCore import *
    from PySide2.QtUiTools import *
    from PySide2.QtGui import *
    from PySide2.QtWidgets import *
except:
    from PyQt5.QtCore import *
    from PyQt5.QtUiTools import *
    from PyQt5.QtGui import *
    from PyQt5.QtWidgets import *

def add_items(combo_box, items, duplicates_allowed=False, clear=False, string_split_character=None):
    """
    Convenience function to add items to a combobox

    :param combo_box: your QComboBox
    :param items: <string> | <list> a single string item ("option"), a list (["option_1", "option_2"]), or a string that can be split on a character like a comma separated string ("option_1, option_2")
    :param duplicates_allowed: <bool> whether or not allow duplicates in the combobox
    :param clear: <bool> clear the comboBox before adding stuff
    :param string_split_character: <string> split the string on this character (eg: ","), the result is the list of items that get added to the combobox
    :return:
    """
    if not isinstance(items, (list, tuple)):
        if isinstance(items, str) and string_split_character is not None and string_split_character in items:
            tmp_list = [part.strip() for part in items.split(string_split_character)]
            items = tmp_list
        elif isinstance(items, str) and items == "":
            return
        else:
            items = [items]

    if clear is True:
        combo_box.clear()

    for item in items:
        if duplicates_allowed is True:
            combo_box.addItem(item)
        else:
            if combo_box.findText(item, Qt.MatchExactly) == -1 and item is not None:
                combo_box.addItem(item)


def remove_items(combo_box, items, string_split_character=None):
    """
    Convenience function to remove items from a combo box

    :param combo_box: your QComboBox
    :param items: <string> | <list> a single string item ("option"), a list (["option_1", "option_2"]), or a string that can be split on a character like a comma separated string ("option_1, option_2")
    :param string_split_character: <string> split the string on this character (eg: ","), the result is the list of items that get added to the combobox
    :return:
    """
    if not isinstance(items, (list, tuple)):
        if isinstance(items, str) and string_split_character is not None and string_split_character in items:
            tmpList = [part.strip() for part in items.split(string_split_character)]
            items = tmpList
        else:
            items = [items]

    for item in items:
        index = combo_box.findText(item, Qt.MatchExactly)
        if index != -1:
            combo_box.removeItem(index)


def set_to_item(combo_box, item_text, reload=False):
    """
    Convenience function to set the text of a combobox to a certain item of the combobox

    :param combo_box: your QComboBox
    :param item_text: <string> text you're looking for
    :param reload: <bool> if True will reload the combobox, even if the text you're setting it to is the current text
    :return:
    """
    if reload is True:
        combo_box.setCurrentIndex(-1)

    index = combo_box.findText(item_text, Qt.MatchExactly)
    if index != -1:
        combo_box.setCurrentIndex(index)
    else:
        raise ValueError("%s is not an item in %s" % (item_text, combo_box.objectName()))


def get_all_items(combo_box):
    """
    Gets all the items in a combobox as a list. Will raise an IndexError if there are not

    items in the combobox
    :param combo_box: your QComboBox
    :return: items of the QComboBox
    """
    items = []
    for item in [combo_box.itemText(i) for i in
                 range(combo_box.count())]:
        if item is not None:
            items.append(item)

    return items


try:
    from PySide2.QtCore import *
    from PySide2.QtUiTools import *
    from PySide2.QtGui import *
    from PySide2.QtWidgets import *
except:
    from PyQt5.QtCore import *
    from PyQt5.QtUiTools import *
    from PyQt5.QtGui import *
    from PyQt5.QtWidgets import *

from functools import partial




def clear_layout(layout):
    """
    Clears a layout of all widgets in it
    #https://stackoverflow.com/questions/4528347/clear-all-widgets-in-a-layout-in-pyqt

    :param layout: <QLayout>
    :return: None
    """
    if layout is not None:
        while layout.count():
            child = layout.takeAt(0)
            if child.widget() is not None:
                child.widget().deleteLater()
            elif child.layout() is not None:
                clear_layout(child.layout())


def delete_window(object=None, name=None):
    """
    Deletes a a window. Pass in either the object or the object name

    :param object: <QObject> you want to delete
    :param name: <string> objectName of the object you want to delete
    :param delete_later: <bool>
    :return:
    """
    app = QApplication.instance()
    for widget in app.topLevelWidgets():
        if object is not None:
            if str(widget.__class__) == str(object.__class__):
                widget.deleteLater()
                widget.close()

        elif name is not None:
            if str(widget.__class__) == name:
                widget.deleteLater()
                widget.close()

def blur_widget(widget, including_children=True, radius=10):
    """
    Blurs a QWidget

    :param widget: <QWidget>
    :param including_children: <bool> self exaplanatory
    :param radius: <int> how much blurriness you want
    :return:
    """
    blur = QGraphicsBlurEffect()
    blur.setBlurRadius(radius)

    widget.setGraphicsEffect(blur)

    if including_children:
        for child in widget.findChildren(QWidget):
            try:
                print(child)
                child.setGraphicsEffect(blur)
            except:
                print("can't do it!")
                pass

class QHLine(QFrame):
    def __init__(self):
        """
        Adds a horizontal line, like you can do in QtDesigner

        """
        super(QHLine, self).__init__()
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)


class QVLine(QFrame):
    def __init__(self):
        """
        Adds a horizontal line, like you can do in QtDesigner

        """
        super(QVLine, self).__init__()
        self.setFrameShape(QFrame.VLine)
        self.setFrameShadow(QFrame.Sunken)



def clickable(widget):
    """
    Makes a QWidget that's not clickable, clickable anyway :)
    # https://wiki.python.org/moin/PyQt/Making%20non-clickable%20widgets%20clickable

    use:
    my_label = QLabel("Click me!")
    clickable(my_label).connect(identify)

    def identify():
        print("Got clicked!")

    :param widget: <QWidget> you want to make clickable
    :return:
    """
    class Filter(QObject):
        clicked = Signal()
        dropped = Signal()

        def eventFilter(self, obj, event):
            if obj == widget:
                if event.type() == QEvent.MouseButtonRelease:
                    if obj.rect().contains(event.pos()):
                        if event.button() == Qt.LeftButton:
                            self.clicked.emit()
                            # The developer can opt for .emit(obj) to get the object within the slot.
                            return True
            return False

    filter = Filter(widget)
    widget.installEventFilter(filter)
    return filter.clicked


def get_all_widgets_in_layout(layout):
    """
    Recursively gets all the items of a layout

    :param layout: <QLayout>
    :return:
    """

    def get_children(item):
        try:
            for index in range(item.count()):
                yield item.itemAt(index).widget()

                for index_2 in get_children(item.itemAt(index)):
                    yield index_2
        except:
            pass

    children = get_children(layout)
    return [child for child in children if child is not None]



class LineEditWithClearButton(QLineEdit):
    """
    QLineEdit with a X button at the end
    #https://stackoverflow.com/a/12468386

    """
    def __init__(self, parent=None):
        super(LineEditWithClearButton, self).__init__(parent)

        self.clear_button = QToolButton(self)

        self.clear_button.setIcon(self.style().standardIcon(getattr(QStyle, "SP_DialogCancelButton")))

        self.clear_button.setStyleSheet('border: 0px; padding: 0px;')
        self.clear_button.setCursor(Qt.PointingHandCursor)
        self.clear_button.clicked.connect(self.clear)

        frame_width = self.style().pixelMetric(QStyle.PM_DefaultFrameWidth)
        button_size = self.clear_button.sizeHint()

        self.setStyleSheet('QLineEdit {padding-right: %dpx; }' % (button_size.width() + frame_width + 1))
        self.setMinimumSize(max(self.minimumSizeHint().width(), button_size.width() + frame_width*2 + 2),
                            max(self.minimumSizeHint().height(), button_size.height() + frame_width*2 + 2))

        self.textChanged.connect(self.check_clear_button_enabled)
        self.check_clear_button_enabled()

    def check_clear_button_enabled(self):
        if self.text():
            self.clear_button.setEnabled(True)
        else:
            self.clear_button.setEnabled(False)

    def resizeEvent(self, event):
        button_size = self.clear_button.sizeHint()
        frame_width = self.style().pixelMetric(QStyle.PM_DefaultFrameWidth)
        self.clear_button.move(self.rect().right() - frame_width - button_size.width(), (self.rect().bottom() - button_size.height() + 3)/2)
        super(LineEditWithClearButton, self).resizeEvent(event)


class EasyUI(QMainWindow):
    def __init__(self, window_title="Easy UI Window", horizontal_layout=False, vertical_layout=True, parent=None):
        """
        Make a QtMainwindow that you easily can add buttons to. Helpful if you want to make a quick tool without
        having to code a window from scratch or use a .ui file

        :param window_title: <string> optional window title
        :param: horizontal_layout: <bool> set to True to have to buttons horizontal
        :param: vertical_layout: <bool> True by default, puts the buttons underneath eachother
        :param parent: <QMainWindow> like Maya's main window
        """
        super(EasyUI, self).__init__(parent)

        self.central_widget = QWidget(self)

        self.setCentralWidget(self.central_widget)
        self.setWindowTitle(window_title)
        self.setObjectName(filter(str.isalpha, window_title))

        if horizontal_layout is True:
            self.horizontal_layout = QHBoxLayout(self.central_widget)
            self.layout = self.horizontal_layout
        else:
            self.vertical_layout = QVBoxLayout(self.central_widget)
            self.layout = self.vertical_layout

    def add_button(self, caption, command, *args, **kwargs):
        """
        Adds a button to the window

        :param caption: Text on the button
        :param command: Function that needs to be called
        :param tooltip: tooltip to add to the button
        :param height: fixed height of the button
        :return: The button that was created


        Example:
        def print_something(mandatory, **kwargs):
            print mandatory
            for key, value in kwargs.iteritems():
                if key == "optional":
                    if value != "":
                        print value

        def print_whaa():
            print "whaa"

        ez_win = qutils.EasyUI(window_title="My super tool!", horizontal_layout=True)
        ez_win.add_button("Print Something Button", print_something, "this is the mandatory message", button_tooltip="this is the tip of the tool", message="this is the message")
        ez_win.add_button("Print Whaa Button", print_whaa, button_height=25)
        ez_win.show()


        """
        if caption is None or command is None:
            raise RuntimeError("Caption or command have to be set!")

        tooltip = kwargs.get("button_tooltip")
        height = kwargs.get("button_height")

        button = QPushButton(self)
        button.setText(caption)
        if not kwargs:
            button.clicked.connect(command)
        else:
            button.clicked.connect(partial(command, *args, **kwargs))
        button.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)

        if tooltip:
            button.setToolTip(tooltip)

        if height:
            button.setFixedHeight(height)

        self.layout.addWidget(button)

        return button

    def add_generic_widget(self, widget):
        self.layout.addWidget(widget)

    def show_ui(self):
        """
        Shows the window

        :return:
        """
        self.show()

def get_named_parent_widget(source_widget, parent_widget_object_name):
    """
    Returns the parent widget with a given name of source_widget

    :param source_widget: <QWidget>
    :param parent_widget_object_name: <string> objectName of the parent widget you're looking for
    :return:
    """
    parent_widget = source_widget.parentWidget()
    if parent_widget is not None and parent_widget.objectName() != parent_widget_object_name:
        return get_named_parent_widget(parent_widget, parent_widget_object_name)
    return parent_widget




from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='ez_qt',
    version='1.0.4',
    packages=['ez_qt'],
    url='https://www.github.com/nielsvaes/ez_qt',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    author='Niels Vaes',
    author_email='nielsvaes@gmail.com',
    description='Helper functions and classes to work with PySide2/PyQt5'
)
